'use strict';

var _ = require('lodash');
var WebSQL = require('kinda-web-sql');
var CordovaSQLite = require('kinda-cordova-sqlite');
var SQLStore = require('kinda-sql-store');

var WebSQLStore = SQLStore.extend('WebSQLStore', function () {
  this.creator = function () {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    if (!options.url) throw new Error('WebSQLStore url is missing');
    if (_.startsWith(options.url, 'websql:')) {
      options.name = options.url.substr('websql:'.length);
      this.connection = WebSQL.create(options);
      this.transactionsAreDisabled = true;
    } else if (_.startsWith(options.url, 'sqlite:')) {
      options.name = options.url.substr('sqlite:'.length);
      this.connection = CordovaSQLite.create(options);
    } else {
      throw new Error('invalid url');
    }
    this.store = this;
    this.setOptions(options);
  };

  this.initializeDatabase = function callee$1$0() {
    var sql;
    return regeneratorRuntime.async(function callee$1$0$(context$2$0) {
      while (1) switch (context$2$0.prev = context$2$0.next) {
        case 0:
          if (!this.store.databaseHasBeenInitialized) {
            context$2$0.next = 2;
            break;
          }

          return context$2$0.abrupt('return');

        case 2:
          sql = 'CREATE TABLE IF NOT EXISTS `pairs` (';

          sql += '`key` longblob NOT NULL, ';
          sql += '`value` longblob, ';
          sql += 'PRIMARY KEY (`key`)';
          sql += ');';
          context$2$0.next = 9;
          return regeneratorRuntime.awrap(this.connection.query(sql));

        case 9:
          this.store.databaseHasBeenInitialized = true;

        case 10:
        case 'end':
          return context$2$0.stop();
      }
    }, null, this);
  };

  this.transaction = function callee$1$0(fn, options) {
    return regeneratorRuntime.async(function callee$1$0$(context$2$0) {
      while (1) switch (context$2$0.prev = context$2$0.next) {
        case 0:
          if (!(this.transactionsAreDisabled || this.isInsideTransaction)) {
            context$2$0.next = 4;
            break;
          }

          context$2$0.next = 3;
          return regeneratorRuntime.awrap(fn(this));

        case 3:
          return context$2$0.abrupt('return', context$2$0.sent);

        case 4:
          context$2$0.next = 6;
          return regeneratorRuntime.awrap(this.initializeDatabase());

        case 6:
          context$2$0.next = 8;
          return regeneratorRuntime.awrap(this.connection.transaction((function callee$2$0(tr) {
            var transaction;
            return regeneratorRuntime.async(function callee$2$0$(context$3$0) {
              while (1) switch (context$3$0.prev = context$3$0.next) {
                case 0:
                  transaction = Object.create(this);

                  transaction.connection = tr;
                  context$3$0.next = 4;
                  return regeneratorRuntime.awrap(fn(transaction));

                case 4:
                  return context$3$0.abrupt('return', context$3$0.sent);

                case 5:
                case 'end':
                  return context$3$0.stop();
              }
            }, null, this);
          }).bind(this), options));

        case 8:
          return context$2$0.abrupt('return', context$2$0.sent);

        case 9:
        case 'end':
          return context$2$0.stop();
      }
    }, null, this);
  };

  Object.defineProperty(this, 'isInsideTransaction', {
    get: function get() {
      return this !== this.store;
    }
  });
});

module.exports = WebSQLStore;